function S = sdh_calculate(D, M, varargin)

% SDH_CALCULATE Calculate Smoothened Data-Histogram of data on SOM
%
%  S = sdh_calculate(sData, sMap, [argID, value, ...])
% 
%   S = sdh_calculate(sData, sMap);
%   S = sdh_calculate(sData, sMap, 'interp.ntimes', 2, 'frame', 'off');
%
%   Input and output arguments ([]'s are optional): 
%     sData   (struct) SOM Toolbox data structure
%             (matrix) training data, size dlen x dim
%     sMap    (struct) SOM Toolbox map structure (rectangular lattice)
%             (matrix) codebook 
%    [argID,  (string) See below.  The values which are unambiguous can 
%     value]  (scalar) be given without the preceeding argID.
%
%     S       (struct) smoothened data histogram structure.
%
%   Here are the valid argument IDs and corresponding values. The values 
%   which are unambiguous (marked with '*') can be given without the
%   preceeding argID.
%     'msize'          (vector) map size, only necessary when sMap is not map struct
%     'spread'         (scalar) SDH spread parameter, default = 3.
%     'method'        *(string) SDH calculation method: '1/n', 'knn', or 'ranking' (default)
%     'frame'         *(string) add frame to round off SDH: 'off' or 'on' (default)
%     'interp.ntimes'  (scalar) expands the matrix by interleaving interpolates
%                                between every element, working recurively for ntimes.
%                                see INTERP2 for details. default is 2, when set to 0 
%                                no intermediate values are calculated.
%     'interp.method' *(string) interpolation method: 'nearest', 'linear', 'spline', 
%                                or 'cubic' (default). see INTERP2 for details.
%
%  See also SDH_VISUALIZE, INTERP2.

% elias 25/04/2002

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% check arguments

% defaults
spread = 3;
method = 'ranking';
frame = 'on';
interp.ntimes = 2;
interp.method = 'cubic';
msize = [];

if isstruct(D) 
   D = D.data;
end
[vecs dims] = size(D);

if isstruct(M) 
   msize = M.topol.msize;
   M = M.codebook;
end

% varargin
i=1; 
while i<=length(varargin), 
   argok = 1; 
   if ischar(varargin{i}), 
      switch varargin{i}, 
         % argument IDs
      case 'msize',         i=i+1; msize = varargin{i}; 
      case 'spread',        i=i+1; spread = varargin{i}; 
      case 'method',        i=i+1; method = varargin{i}; 
      case 'frame',         i=i+1; frame = varargin{i}; 
      case 'interp.ntimes', i=i+1; interp.ntimes = varargin{i}; 
      case 'interp.method', i=i+1; interp.method = varargin{i};
         % unambiguous values
      case {'on','off'}, frame = varargin{i};
      case {'ranking','1/n','knn'}, method = varargin{i}; 
      case {'nearest','linear','cubic','spline'}, interp.method = varargin{i};
      otherwise argok=0; 
      end
   else
      argok = 0; 
   end
   if ~argok, 
      disp(['(sdh_calculate) Ignoring invalid argument #' num2str(i+1)]); 
   end
   i = i+1; 
end

if isempty(msize),
   error('(sdh_calculate) msize not specified!')
end

S.type = 'sdh';
S.msize = msize;
S.frame = frame;
S.method = method;
S.spread = spread;
S.interp.ntimes = interp.ntimes;
S.interp.method = interp.method;
%% To be filled in coming versions:
% S.data_centers: to create html image maps
% S.peeks: to label mountains

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% START
rows = msize(1);
cols = msize(2);

Dist = repmat(sum(M.^2,2),1,vecs)-M*2*D'; % munits x vecs
m = zeros(rows*cols,1);

switch method,
case 'ranking', c = (spread:-1:1)';
case 'knn', c = ones(spread,1);
case '1/n', c = (1./(1:spread))';
otherwise error('(shd_calculate) unknown method.');
end

for i=1:vecs,
   [d idx] = sort(Dist(:,i));
   idx = idx(1:spread); 
   m(idx) = m(idx) + c;
end
m = reshape(m,rows,cols);

if strcmp(frame,'on'), 
    m = sdh_addframe(m,0.1);
end

S.matrix = interp2(m,interp.ntimes,interp.method);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
