function sdh_visualize(S, varargin)

% SDH_VISUALIZE Visualize Smoothened Data-Histogram
%
%  sdh_visualize(S, [argID, value, ...])
% 
%   S = sdh_calculate(sData, sMap);
%   sdh_visualize(S);
%   sdh_visualize(S,'type','contour');
%   sdh_visualize(S,'grid','on');
%
%   Input arguments ([]'s are optional): 
%     S       (struct) Smoothened Data Histogram structure, see sdh_calculate
%    [argID,  (string) See below.  The values which are unambiguous can 
%     value]  (scalar) be given without the preceeding argID.
%
%   Here are the valid argument IDs and corresponding values. The values 
%   which are unambiguous (marked with '*') can be given without the
%   preceeding argID.
%     'type'   *(string) 'continuous' (default), 'contour'.
%     'grid'   *(string) 'on', 'nodes', or 'off' (default). Depicts 
%                        relationship between underlining map and visualization.
%     'levels'  (scalar) contour levels, default = 4.
%     'subplot' (scalar) plot image into this subplot (handle)
%     'labels'  (cell)   labels (see sMap.labels)
%
%  See also SDH_CACULATE.

% elias 25/04/2002

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% check arguments

% defaults
FONTSIZE = 7;
s = [];
type = 'continuous';
grid = 'off';
labels = {};
levels = 4;
titel = ['n = ',num2str(S.spread)];

if ~isstruct(S) |  ~isfield(S,'type') | ~strcmp(S.type,'sdh'),
    error('(sdh_visualize) first argument is not SDH structure.');
end

% varargin
i=1; 
while i<=length(varargin), 
    argok = 1; 
    if ischar(varargin{i}), 
        switch varargin{i}, 
            % argument IDs
        case 'title',         i=i+1; titel = varargin{i}; 
        case 'type',          i=i+1; type = varargin{i}; 
        case 'grid',          i=i+1; grid = varargin{i}; 
        case 'levels',        i=i+1; levels = varargin{i}; 
        case 'subplot',       i=i+1; s = varargin{i}; 
        case 'labels',        i=i+1; labels = varargin{i}; 
            % unambiguous values
        case {'on','nodes','off'}, grid = varargin{i};
        case {'continuous','contour'}, type = varargin{i}; 
        otherwise argok=0; 
        end
    elseif isnumeric(varargin{i}),
        level = varargin{i};
    else
        argok = 0; 
    end
    if ~argok, 
        disp(['(sdh_visualize) Ignoring invalid argument #' num2str(i+1)]); 
    end
    i = i+1; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% START

if isempty(s),
    f = figure;
    s = subplot(1,1,1);
    set(f,'name',titel);
else
    subplot(s);
end
title(titel);

% visualize SDH
switch type, 
case 'continuous', 
    pcolor(S.matrix); shading interp;
    sdh_colormap('islands');
case 'contour',
    contourf(S.matrix,levels);
    colormap(flipud(1-gray/2));
otherwise, 
    error('(sdh_visualize) unsupported type!');
end

if strcmp(grid,'on'), % use grid so that cells correspond to map units
    hold on; 
    
    x = S.msize(2);
    y = S.msize(1);
    c = 2^S.interp.ntimes;
    
    if strcmp(S.frame,'on'), f = 1;
    else f = 0;
    end
    
    o=f*c-c/2+1;
    
    for i=1:x+1, 
        plot([o+(i-1)*c, o+(i-1)*c],[o, o+c*y],'-k');
    end
    for i=1:y+1, 
        plot([o, o+c*x],[o+(i-1)*c, o+(i-1)*c],'-k');
    end
elseif strcmp(grid,'nodes'), % use grid so that grid nodes correspond to map units
    hold on; 
    
    x = S.msize(2);
    y = S.msize(1);
    c = 2^S.interp.ntimes;
    
    if strcmp(S.frame,'on'), f = 1;
    else f = 0;
    end
    
    o=f*c+1;
    
    for i=1:x, 
        plot([o+(i-1)*c, o+(i-1)*c],[o, o+c*(y-1)],'-k');
    end
    for i=1:y, 
        plot([o, o+c*(x-1)],[o+(i-1)*c, o+(i-1)*c],'-k');
    end
    
    for i=1:y,
        for j=1:x,
            plot(o+(j-1)*c, o+(i-1)*c,'k.');
        end
    end
end

if ~isempty(labels),
    hold on; 
    
    x = S.msize(2);
    y = S.msize(1);
    c = 2^S.interp.ntimes;
    
    if strcmp(S.frame,'on'), f = 1;
    else f = 0;
    end
    
    o=f*c+1;
    
    k=1;
    for j=1:x,
        for i=1:y,
            
            str = labels(k,find(strcmp(labels(k,:),'')==0));
            
            OFFSET = [0.025, 0.03, 0.035, 0.04];
            for offis=1:length(OFFSET),
                text(o+(j-1)*c-OFFSET(offis),o+(i-1)*c-OFFSET(offis),...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                text(o+(j-1)*c+OFFSET(offis),o+(i-1)*c-OFFSET(offis),...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                text(o+(j-1)*c-OFFSET(offis),o+(i-1)*c+OFFSET(offis),...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                text(o+(j-1)*c+OFFSET(offis),o+(i-1)*c+OFFSET(offis),...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                
                text(o+(j-1)*c,o+(i-1)*c-OFFSET(offis),...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                text(o+(j-1)*c+OFFSET(offis),o+(i-1)*c,...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                text(o+(j-1)*c-OFFSET(offis),o+(i-1)*c,...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
                text(o+(j-1)*c,o+(i-1)*c+OFFSET(offis),...
                    str,'fontsize',FONTSIZE,'horizontalalignment','center')
            end            
            
            h=text(o+(j-1)*c, o+(i-1)*c, ...
                str,'fontsize',FONTSIZE,'horizontalalignment','center','color','w'); 
            k=k+1;
        end
    end
end

set(s,'ydir','reverse');
set(s,'xtick',[-realmax realmax]);
set(s,'ytick',[-realmax realmax]);

set(f,'renderer','painter')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%